# vim: fileencoding=utf-8
#
from .base_format import BaseFormat
from collections import OrderedDict
from pyparsing import Word, Literal, printables, originalTextFor, ZeroOrMore,\
                      OneOrMore, ParseException, restOfLine,\
                      pyparsing_unicode, Group, Optional


class OpenRCFormat(BaseFormat):
    def __init__(self, document_text: str, ignore_comments=False):
        processing_methods = [self._parse_comment_line,
                              self._parse_parameter_line,
                              self._parse_to_delete_line]

        super().__init__(processing_methods)
        self._ignore_comments = ignore_comments
        self._comments_processing = True
        self._format = 'openrc'

        self._last_comments_list = []
        self._initialize_parser()

        if document_text == '':
            self._document_dictionary = OrderedDict()
        else:
            document_lines = self._get_list_of_logic_lines(document_text)
            self._lines_to_dictionary(document_lines)

    def _initialize_parser(self):
        parameter_name = Word(printables, excludeChars='=')

        parameter_value = originalTextFor(OneOrMore(Word(printables)))

        action_symbols = (Literal('!') | Literal('-'))

        self._parameter_line = (Group(Optional(action_symbols,
                                               default='')('action')
                                + parameter_name('name'))('parameter_name')
                                + Literal('=').suppress()
                                + parameter_value('parameter_value'))

        self._parameter_to_delete = (Group(action_symbols('action')
                                           + parameter_name('name')
                                           )('parameter_name')
                                     + restOfLine.suppress())

        self._comment_line = originalTextFor(
                             Literal('#')
                             + ZeroOrMore(Word(printables
                                               + pyparsing_unicode.alphanums))
                                            )('comment')

    def _parse_parameter_line(self, line):
        try:
            self._item_to_add = OrderedDict()
            parsing_result = self._parameter_line.parseString(line)

            key_value = (parsing_result.parameter_name.action,
                         parsing_result.parameter_name.name.lower())
            parameter_value = parsing_result.parameter_value

            parameter_value = self._last_comments_list + [parameter_value]
            self._last_comments_list = []

            self._item_to_add[key_value] = parameter_value
            self._ready_to_update = True
            self._match = True
        except ParseException:
            return

    def _parse_to_delete_line(self, line):
        try:
            self._item_to_add = OrderedDict()
            parsing_result = self._parameter_to_delete.parseString(line)

            if parsing_result.parameter_name.action == '-':
                return

            key_value = (parsing_result.parameter_name.action,
                         parsing_result.parameter_name.name.lower())
            parameter_value = self._last_comments_list
            self._last_comments_list = []

            self._item_to_add[key_value] = parameter_value
            self._ready_to_update = True
            self._match = True
        except ParseException:
            return

    def _parse_comment_line(self, line):
        try:
            result = self._comment_line.parseString(line)
            self._match = True

            if not self._ignore_comments:
                self._last_comments_list.append(result.comment)
        except ParseException:
            return
